#define OPTIMIZE_ROM_CALLS
//#define RETURN_VALUE
#define ENABLE_ERROR_RETURN
#define SAVE_SCREEN

#include <tigcclib.h>
typedef unsigned char byte;

byte grid[9][9];
byte restricted[9][9][9];

void defineRestrictedMatrix();
void dispPossibilities(byte squ);
byte getBox(byte squ);
byte getCol(byte squ);
byte getGridNumber(byte squ);
byte getPossibilities(byte squ);
byte getRow(byte squ);
byte newGrid();
void processAllDefinedSquares();
void processBox(byte box);
void processCol(byte col);
void processDefinedSquare(byte squ);
void processEverything();
void processOneOccuranceInACol(byte col);
void processOneOccuranceInARow(byte row);
void processRow(byte row);
void processSquares();
void substituteAllSecureSquares();
void updateIO();

/**
	*Main function for solving a given Sudoku puzzle
*/
void _main(void)
{
	byte r;
	defineRestrictedMatrix();
	if(!newGrid())
		exit(40);
	puzzle1();
	updateIO();
	//processAllDefinedSquares();
	for(r=0; r<100; r++)
	{
		processAllDefinedSquares();
		processSquares();
		//processEverything();
		//substituteAllSecureSquares();
	}
	updateIO();
}

/**
	*Restricts the possible values for the squares (1-9)
*/
void defineRestrictedMatrix()
{
	byte r,c,n;
	for(r=0; r<9; r++)
		for(c=0; c<9; c++)
			for(n=1; n<=9; n++)
				restricted[r][c][n-1] = n;
}

/**
	*If a square is defined, set all its possibilities to zero
	*except the number that it already is.
	*
	*Input square 1-81
*/
void processDefinedSquare(byte squ)
{
	byte row=getRow(squ)-1, col=getCol(squ)-1, i, num=getGridNumber(squ);
	if(num != 0)
	{
		for(i=0; i<9; i++)
			restricted[row][col][i] = 0;
		restricted[row][col][num-1] = num;
	}
}

/**
	*Processes all defined squares.
*/
void processAllDefinedSquares()
{
	byte i=1;
	for(; i<=81; i++)
		processDefinedSquare(i);
}

void processSquares()
{
	byte i,j,k,n,t,c=0;
	for(i=0; i<9; i++)
	{
		for(j=0; j<9; j++)
		{
			n = grid[i][j];
			if(n != 0)
			{
				for(k=0; k<9; k++)
				{
					restricted[i][k][n-1] = 0;
					restricted[k][j][n-1] = 0;
					restricted[i][j][k] = 0;
				}				
			}
			else
			{
				for(k=0; k<9; k++)
					if(restricted[i][j][k] != 0)
					{
						t = k;
						++c;
					}
				if(c == 1)
					grid[i][j] = t+1;
			}
		}
		c = 0;
		processBox(i+1);
	}
}

/**
	*Processes every row, column, and major box on the grid.
*/
void processEverything()
{
	byte i=1;
	for(; i<=9; i++)
	{
		//processRow(i);
		//processCol(i);
		//processBox(i);
		//processOneOccuranceInACol(i);
		//processOneOccuranceInARow(i);
		
	}
}

void dispPossibilities(byte squ)
{
	byte r,row=getRow(squ)-1,col=getCol(squ)-1;
	for(r=0; r<9; r++)
		printf("%d,",restricted[row][col][r]);
	printf("__");
	ngetchx();
}

/**
	*If a certain number can only exist in one square in a row,
	*this method handles that.
	*
	*Input row 1-9
*/
void processOneOccuranceInARow(byte row)
{
	row--;
	byte i,j,last=0,count=0;
	for(i=1; i<=9; i++)	//i = the value that only exists once that we are searching for 
	{
		for(j=0; j<9; j++)	//j = represents the different columns we are looping through
		{
			if(grid[row][j]==0 && restricted[row][j][i-1]!=0)
			{
				count++;
				last = j;
			}
		}
		if(count == 1)
			grid[row][last] = i;
		count = 0;
	}
}

/**
	*If a certain number can only exist in one square in a column,
	*this method handles that.
	*
	*Input column 1-9
*/
void processOneOccuranceInACol(byte col)
{
	col--;
	byte i,j,last=0,count=0;
	for(i=1; i<=9; i++)	//i = the value (for which we are searching) that only exists once
	{
		for(j=0; j<9; j++)	//j = represents the different rows we are looping through
		{
			if(grid[j][col]==0 && restricted[j][col][i-1]!=0)
			{
				count++;
				last = j;
			}
		}
		if(count == 1)
			grid[last][col] = i;
		count = 0;
	}
}

/**
	*Go through every square in the grid. If there is only one possibility
	*for any of those squares, assign that value to that spot in the grid.
	*Clear that value out of all other squares in that row, column, and major
	*box by calling the appromriate methods.
	*
	*Input square 1-81
	*Output the number of values substituted
*/
void substituteAllSecureSquares()
{
	byte e,n,r,c;
	for(e=1; e<=81; e++)
	{
		n = getPossibilities(e);
		if(0<n && n<10)
		{
			r = getRow(e);
			c = getCol(e);
			grid[r-1][c-1] = n;
		}
	}
}

/**
	*If there is only one possibility for the given square,
	*return that possibility as an unsigned char.
	*
	*If there is more than one possibility, return zero.
	*
	*If there are no possibilities, return ten.
	*
	*Input square 1-81
*/
byte getPossibilities(byte squ)
{
	byte row=getRow(squ), col=getCol(squ), c=0, i=0, re=0, temp;
	if(grid[--row][--col] != 0)
		return grid[row][col];
	for(; i<9; i++)
	{
		temp = restricted[row][col][i];
		if(temp!=0)
		{
			c++;
			re = temp;
		}
	}
	switch(c)
	{
		case 0: return 10;
		case 1: return re;
		default:return 0;
	}
}

/**
	*Initialize a new grid from the 81 arguments (or zero arguments)
	*that are passed into the function from the
	*TI-OS home screen.
*/
byte newGrid()
{
	byte r,c,re = ArgCount();
	if(re>81)
		ER_throw(40);
	re = TRUE;
	ESI q = top_estack;
	for(r=0; r<9; r++)
		for(c=0; c<9; c++)
		{
			grid[r][c] = GetIntArg(q);
			if(grid[r][c]>9)
				re = FALSE;
		}
	return re;
}

/**
	*Clear the screen.
	*Display the grid in a crude but understandable format
*/
void updateIO()
{
	FontSetSys(0);
	byte r, c;
	clrscr();
	for(r=0; r<9; r++)
	{
		for(c=0; c<9; c++)
			{
				if((c+1)%3 == 1)
					printf(" | ");
				if(grid[r][c] != 0)
					printf(" %d ",grid[r][c]);
				else
					printf("    ");
			}
			if((r+1) % 3 == 0)
				printf("|\n-----------------------\n");
			else
				printf("|\n");
	}
	ngetchx();
}

/**
	*In any given row, if there is a square in that row
	*that is defined, remove that possibility from every
	*other square in that row.
	*
	*Input row 1-9
*/
void processRow(byte row)
{
	byte c,n,j;
	for(c=0; c<9; c++)
	{
		n = grid[row-1][c];
		if(n != 0)
			for(j=0; j<9; j++)
				restricted[row-1][j][n-1] = 0;
	}
}

/**
	*In any given column, if there is a square in that column
	*that is defined, remove that possibility from every
	*other square in that column.
	*
	*Input column 1-9
*/
void processCol(byte col)
{
	byte r,n,j;
	for(r=0; r<9; r++)
	{
		n = grid[r][col-1];
		if(n != 0)
			for(j=0; j<9; j++)
				restricted[j][col-1][n-1] = 0;
	}
}

/**
	*In any given box, if there is a square in that box
	*that is defined, remove that possibility from every
	*other square in that box.
	*
	*Input box 1-9
*/
void processBox(byte box)
{
	byte first=0;
	switch(box)
	{
		case 1: first=1; break;
		case 2: first=4; break;
		case 3: first=7; break;
		case 4: first=28; break;
		case 5: first=31; break;
		case 6: first=34; break;
		case 7: first=55; break;
		case 8: first=58; break;
		case 9: first=61;
	}
	byte list[9] = {first,first+1,first+2,first+9,first+10,first+11,first+18,first+19,first+20},i,j,t,n,g;
	for(i=0; i<9; i++)
	{
		n = list[i];
		g = getGridNumber(n);
		if(g != 0)
		{
			for(j=0; j<9; j++)
			{
				t = list[j];
				restricted[getRow(t)-1][getCol(t)-1][g-1] = 0;
			}
		}
	}
}

/**
	*Find which of the nine major boxes a given square lies in.
	*
	*Input square 1-81
	*Output box 1-9
*/
byte getBox(byte squ)
{
	byte col = getCol(squ), row = getRow(squ);
	switch(col)
	{
		case 1 ... 3: col=1; break;
		case 4 ... 6: col=2; break;
		case 7 ... 9: col=3;
	}
	switch(row)
	{
		case 1 ... 3: row=1; break;
		case 4 ... 6: row=2; break;
		case 7 ... 9: row=3;
	}
	switch(row)
	{
		case 1: return col;
		case 2: return 3 + col;
		case 3: return 6 + col;
	}
	return 0;
}

/**
	*Get the column that a given square lies in.
	*
	*Input square 1-81
	*Output column 1-9
*/
byte getCol(byte squ)
{
	byte re = squ % 9;
	return (re==0) ? 9 : re;
}

/**
	*Get the row that a given square lies in.
	*
	*Input square 1-81
	*Output row 1-9
*/
byte getRow(byte squ)
{
	return squ/9 + (squ%9!=0);
}

/**
	*Get the value of the number in a square
	*
	*Input square 1-81
*/
byte getGridNumber(byte squ)
{
	return grid[getRow(squ)-1][getCol(squ)-1];
}

void puzzle1() //Easy Puzzle 111,532,034
{
	byte puzzle[9][9] =
	 {{0,4,0,3,5,0,0,0,0},
	  {0,0,3,1,0,0,0,8,0},
	  {9,0,1,0,8,0,7,0,3},
	  {0,7,6,0,9,0,0,4,0},
	  {5,0,9,0,4,0,3,0,8},
	  {0,8,0,0,1,0,6,5,0},
	  {1,0,5,0,7,0,2,0,6},
	  {0,6,0,0,0,1,8,0,0},
	  {0,0,0,0,3,5,0,1,0}};
	
	byte i,j;
	for(i=0; i<9; i++)
		for(j=0; j<9; j++)
			grid[i][j] = puzzle[i][j];
}